@@ -1,16 +1,21 @@
 import sys
 import inspect
 import readchar
+import readline as rl
 import argparse
 import datetime
 from pathlib import Path
 from shutil import copytree, rmtree
 
 from vinca.lib.classes import Card
-from vinca.lib.ansi import ansi
+from vinca.lib import ansi
 from vinca.lib import filter
 from vinca.generators import generate, GENERATORS_DICT
 
+# TODO: intelligent scrollback for reviewers and editors
+# TODO: make functions specify the parameters they need and 
+# feed them this explicitly á la FILTER
+
 TODAY = datetime.date.today()
 
 COLORIZE = sys.stdout.isatty()
@@ -19,10 +24,11 @@ QUIT_KEYS = ['q', readchar.key.ESC, '\n', '\r']
 vinca_path = Path(__file__).parent 
 cards_path = vinca_path / 'cards'
 ALL_CARDS = [Card(int(id.name)) for id in cards_path.iterdir()]
+ALL_TAGS = [tag for card in ALL_CARDS for tag in card.tags] # TODO: cache
 
 # ARGUMENT PARSING
-parser = argparse.ArgumentParser(usage=argparse.SUPPRESS)
-subparsers = parser.add_subparsers()
+parser = argparse.ArgumentParser(usage=argparse.SUPPRESS, epilog='Use -h to see help for any command. Visit the manual for usage examples.')
+subparsers = parser.add_subparsers(metavar='')
 def card_type(arg):
 	id = int(arg.split()[0])  # grab the first field of the argument
 	return Card(id)
@@ -49,22 +55,23 @@ def subcommand(*subparser_args, parent=subparsers, alias='', name=None):
 		description = func.__doc__
 		aliases = [alias] if alias else []
 		parser = parent.add_parser(name, help='', description=description, aliases=aliases,
-		                           usage=argparse.SUPPRESS, formatter_class=argparse.RawDescriptionHelpFormatter)
+		                           usage=argparse.SUPPRESS)
 		for args, kwargs in subparser_args:
 			parser.add_argument(*args, **kwargs)
 		parser.set_defaults(func=func)
 		return func
 	return decorator
 
-@subcommand(alias='a')
+@subcommand(arg('-t','--default_tags', nargs='+'), alias='a')
 def add(args): 
-	print(ansi['hide_cursor'],end='')
+	ansi.hide_cursor()
 	print(*[f'{key}\t{generator}' for key, generator in GENERATORS_DICT.items()],sep='\n')
 	k = readchar.readchar()
 	if k not in GENERATORS_DICT: return
 	args.generator = GENERATORS_DICT[k]
 	args.cards = generate(args)
-	print(ansi['move_up_line']*(len(GENERATORS_DICT)+2) + ansi['clear_to_bottom'],end='')
+	ansi.up_line(len(GENERATORS_DICT) + 2)
+	ansi.clear_to_end()
 	browse(args)
 
 @subcommand(OPTIONAL_CARD, OPTIONAL_CARDS, alias='s')
@@ -86,7 +93,7 @@ def statistics(args):
 		print('New', len(new_cards), sep='\t')
 		lines = 3
 	if args.scrollback:
-		print(ansi['move_up_line']*lines,end='')
+		ansi.up_line(lines)
 
 @subcommand(CARD,alias='e')
 def edit(args):
@@ -118,10 +125,56 @@ def review(args):
 			return
 		browse(args, reviewing = True)
 
+@subcommand(arg('tag'), OPTIONAL_CARD, OPTIONAL_CARDS_DEFAULT_NONE)
+def add_tag(args):
+	for card in [args.card] if args.card else args.cards:
+		card.tags += [args.tag]
+
+@subcommand(arg('tag'), OPTIONAL_CARD, OPTIONAL_CARDS_DEFAULT_NONE)
+def remove_tag(args):
+	for card in [args.card] if args.card else args.cards:
+		if args.tag in card.tags:
+			card.tags.remove(args.tag)
+		# TODO do this with set removal
+		card.save_metadata()
+
+@subcommand(OPTIONAL_CARD, OPTIONAL_CARDS_DEFAULT_NONE, alias='t')
+def edit_tags(args):
+	def complete(text, state):
+		for tag in ALL_TAGS:
+			if tag.startswith(text):
+				if not state:
+					return tag
+				else:
+					state -= 1
+	rl.parse_and_bind('tab: complete')
+	rl.set_completer(complete)
+
+	if args.card:
+		pass #TODO lineedit tags with tab complete
+		tags = ' '.join(args.card.tags)
+		rl.set_startup_hook(lambda: rl.insert_text(tags))
+		args.card.tags = input('tags: ').split()
+		if args.scrollback:
+			ansi.up_line()
+		rl.set_startup_hook()
+	elif args.cards:
+		tags_add = input('tags to add: ').split()
+		tags_remove = input('tags to remove: ').split()
+		for tag in tags_add:
+			args.tag = tag
+			add_tag(args)
+		for tag in tags_remove:
+			args.tag = tag
+			remove_tag(args)
+
+	rl.set_completer()
+
 CMD_DICT = {'r': review, 'R': review,
 	    's': statistics, 'S': statistics,
 	    'x': delete, 'X': delete,
-	    'e': edit, 'E': edit_metadata}
+	    'e': edit, 'E': edit_metadata,
+	    't': edit_tags, 'T': edit_tags}
 @subcommand(CARDS, alias='b')
 def browse(args, reviewing = False):
 	args.scrollback = True
@@ -130,22 +183,27 @@ def browse(args, reviewing = False):
 	n = len(cards); sel = 0
 
 	def quit():
-		print(ansi['show_cursor'],end='')
-		print(ansi['line_wrap_on'], end='')
-		print(ansi['clear_to_bottom'],end='')
+		ansi.show_cursor()
+		ansi.line_wrap_on()
+		ansi.clear_to_end()
 		exit()
 	def draw_browser():
-		print(ansi['hide_cursor'],end='')
-		print(ansi['line_wrap_off'], end='')
+		ansi.hide_cursor()
+		ansi.line_wrap_off()
 
 		for i, card in enumerate(cards):
-			x = f'{ansi["red"]}deleted{ansi["reset_color"]} '*card.deleted
-			d = f'{ansi["blue"]}due{ansi["reset_color"]} '*card.due_as_of(args.review_date)
-			hi = ansi['reverse']*(i==sel)
-			print(hi + x + d + str(card) + ansi['reset'])
+			if card.deleted:
+				ansi.crossout()
+			if card.due_as_of(args.review_date):
+				ansi.bold()
+			if i==sel:
+				ansi.highlight()
+			print(card)
+			ansi.reset()
+
 	def redraw_browser():
-		print(ansi['move_up_line']*n, end='') 
-		print(ansi['clear_to_bottom'],end='')
+		ansi.up_line(n)
+		ansi.clear_to_end()
 		draw_browser()
 		
 	draw_browser()
@@ -160,19 +218,23 @@ def browse(args, reviewing = False):
 		if k in QUIT_KEYS:
 				quit()
 		if k in CMD_DICT:
-			args.card = cards[sel] if k in ('x','s','r','R','e','E') else None
+			ansi.show_cursor()
+			args.card = cards[sel] if k in ('x','s','r','R','e','E', 't') else None
 			CMD_DICT[k](args)
+			ansi.hide_cursor()
 			reviewing = (k == 'R' and cards[sel].last_grade != 'exit')
 			if reviewing and sel == n - 1:
 				quit()
 			elif reviewing and sel < n - 1:
 				sel += 1 
 		if k in GENERATORS_DICT:
+			ansi.show_cursor()
 			args.generator = GENERATORS_DICT[k]
 			new_cards = generate(args)
 			cards[sel:sel] = new_cards
 			n = len(cards)
-			print('\n'*(len(new_cards)), end='')
+			ansi.down_line(len(new_cards))
+			ansi.hide_cursor()
 
 		if k not in ('s','S'):
 			redraw_browser()
@@ -213,11 +275,17 @@ def display_filter(args):
 		# feed the keyword args editor=args.editor, due=args.due, 
 		**{param : getattr(args, param) for param in filter_kwargs})
 	for card in matches:
-		d = (ansi['red']+'[deleted]' + ansi['reset_color'] + ' ')*card.deleted
-		print(str(card.id) + f'\t{d}{card}'*(not args.id_only))
+		if args.id_only:
+			print(card.id)
+			continue
+		if card.deleted():
+			ansi.crossout()
+		print(card.id, card, sep='\t')
+		ansi.reset()
 	
 @subcommand()
 def purge(args):
+	''' Permanently delete all cards marked for deletion. '''
 	for card in filter.filter(ALL_CARDS, deleted_only=True):
 		rmtree(card.path)
 
@@ -236,12 +304,16 @@ def imp(args):
 	for new_id,card_path in enumerate(args.import_path.iterdir(), max(old_ids, default=1) + 1):
 		copytree(card_path, cards_path / str(new_id))
 
+
 for alias, generator in GENERATORS_DICT.items():
 	p = subparsers.add_parser(generator, aliases=[alias], help='')
+	p.add_argument('-t','--default_tags',nargs='*',default=[])
 	p.set_defaults(func = generate, generator = generator)
 
 # parse the command line arguments
 parser.set_defaults(cards = [], func = None, review_date = TODAY, scrollback = False, card = None)
+parser._action_groups[0].title = 'commands'
+parser._action_groups.pop()
 args = parser.parse_args()
 if not args.func:
 	args.func = statistics
