@@ -1,5 +1,5 @@
 from vinca.lib import classes
-from vinca.lib.ansi import ansi
+from vinca.lib import ansi
 
 def generate(args):
 	new_card = classes.Card(create=True)
@@ -9,5 +9,5 @@ def generate(args):
 	(new_card.path/'front').write_text(front)
 	(new_card.path/'back').write_text(back)
 	if args.scrollback:
-		print(ansi['move_up_line']*2, end='')
+		ansi.up_line(2)
 	return [new_card]
