@@ -1,7 +1,7 @@
 # A dummy scheduler that always schedules for the next DAY
 import datetime
-TODAY = datetime.date.TODAY()
-DAY = datetime.timedelta(DAYs=1)
+TODAY = datetime.date.today()
+DAY = datetime.timedelta(days=1)
 
 def schedule(card):
 
@@ -19,3 +19,5 @@ def schedule(card):
 		card.due_date = TODAY + card.last_interval * 2 + DAY
 	if card.last_grade == 'easy':
 		card.due_date = TODAY + card.last_interval * 3 + DAY*2
+
+	# TODO refactor using match and case syntax in Python 3.10
